@@ -1,3 +1,4 @@
+import pdb
 import tempfile
 import subprocess
 from pathlib import Path
@@ -51,13 +52,13 @@ def _get_exporter(nbconvert_exporter_name, extension):
     return exporter
 
 
-def _from_ipynb(path_to_nb, exporter):
+def _from_ipynb(path_to_nb, exporter, nbconvert_export_kwargs):
 
     path = Path(path_to_nb)
 
     nb = nbformat.reads(path.read_text(), as_version=nbformat.NO_CONVERT)
     content, _ = nbconvert.export(exporter, nb,
-                                  exclude_input=True)
+                                  **nbconvert_export_kwargs)
 
     path.write_text(content)
 
@@ -91,7 +92,6 @@ class NotebookRunner(Task):
         Kernelspec name to use, if the notebook already includes kernelspec
         data (in metadata.kernelspec), this is ignored, otherwise, the kernel
         is looked up using the jupyter_client.kernelspec.get_kernel_spec
-        function
     nbconvert_exporter_name: str, optional
         Once the notebook is run, this parameter controls whether to export
         the notebook to a different parameter using the nbconvert package,
@@ -114,15 +114,22 @@ class NotebookRunner(Task):
         detect errors before executing the notebook. If the task has
         upstream dependencies an upstream parameter should also be declared
         "upstream = None"
+    nbconvert_export_kwargs : dict
+        Keyword arguments to pass to the nbconvert.export function (this is
+        only used if exporting the output ipynb notebook to another format).
+        You can use this for example to hide code cells using the exclude_input
+        parameter
     """
     PRODUCT_CLASSES_ALLOWED = (File, )
 
-    @requires(['jupyter', 'papermill'], 'NotebookRunner')
+    @requires(['jupyter', 'papermill', 'jupytext'], 'NotebookRunner')
     def __init__(self, source, product, dag, name=None, params=None,
-                 papermill_params=None, kernelspec_name=None,
+                 papermill_params=None, kernelspec_name='python3',
                  nbconvert_exporter_name=None, ext_in=None,
-                 nb_product_key='nb', static_analysis=False):
+                 nb_product_key='nb', static_analysis=False,
+                 nbconvert_export_kwargs=None):
         self.papermill_params = papermill_params or {}
+        self.nbconvert_export_kwargs = nbconvert_export_kwargs or {}
         self.kernelspec_name = kernelspec_name
         self.nbconvert_exporter_name = nbconvert_exporter_name
         self.ext_in = ext_in
@@ -154,61 +161,36 @@ class NotebookRunner(Task):
 
     def develop(self):
         """
-        Opens the notebook (unmodified and in its original location)
-        """
-        # TODO: put debug() contents here and make debug() start an actual
-        # debugging sessions (with the parameters injected)
-        if self.source.loc is None:
-            raise ValueError('Can only use develop in notebooks loaded '
-                             'from files, not from str')
+        Opens the rendered notebook (with injected parameters) and adds a
+        "debugging-settings" cell to the that changes directory to the current
+        active directory. This will reflect conditions when callign
+        `DAG.build()`. This modified notebook is saved in the same location as
+        the source with a "-tmp" added to the filename. Changes to this
+        notebook can be exported to the original notebook after the notebook
+        process is shut down. The "injected-parameters" and
+        "debugging-settings" cells are deleted before saving.
 
-        try:
-            subprocess.call(['jupyter', 'notebook', self.source.loc])
-        except KeyboardInterrupt:
-            print('Jupyter notebook server closed...')
-
-    def debug(self):
-        """
-        Opens the rendered notebook (with a new cell that includes injected
-        parameters and in a temporary location) with debug settings turned on.
-        Changes to this notebook can be exported to the original notebook
-        (the injected parameters cell and cells that turn debugging on are
-        excluded).
+        Notes
+        -----
+        If you modify the source code and call develop again, the source
+        code will be updated only if the `hot_reload option` is turned on.
+        See :class:`ploomber.DAGConfigurator` for details.
         """
         if self.source.loc is None:
             raise ValueError('Can only use develop in notebooks loaded '
                              'from files, not from str')
 
-        # load original notebook content
-        content_original = Path(self.source.loc_rendered).read_text()
+        nb = _read_rendered_notebook(self.source.rendered_nb_str)
 
-        # add debug cells
-        nb = nbformat.reads(content_original, as_version=nbformat.NO_CONVERT)
-        nbformat_v = nbformat.versions[nb.nbformat]
-
-        source = """
-# Debugging settings (this cell will be removed before saving)
-# change the current working directory to the one when .debug() happen
-# to make relative paths work
-from os import chdir
-chdir("{}")
-""".format(Path('.').resolve())
-
-        cell = nbformat_v.new_code_cell(source,
-                                        metadata={'tags':
-                                                  ['debugging-settings']})
-        nb.cells.insert(0, cell)
-
-        # save modified notebook
-        _, tmp = tempfile.mkstemp(suffix='.ipynb')
+        name = self.source.loc.name
+        suffix = self.source.loc.suffix
+        name_new = name.replace(suffix, '-tmp.ipynb')
+        tmp = self.source.loc.with_name(name_new)
         content = nbformat.writes(nb, version=nbformat.NO_CONVERT)
-        Path(tmp).write_text(content)
+        tmp.write_text(content)
 
-        try:
-            # open notebook with injected debugging cell
-            subprocess.call(['jupyter', 'notebook', tmp])
-        except KeyboardInterrupt:
-            print('Jupyter notebook server closed...')
+        # open notebook with injected debugging cell
+        _open_jupyter_notebook(str(tmp))
 
         # read tmp file again, to see if the user made any changes
         content_new = Path(tmp).read_text()
@@ -217,14 +199,8 @@ chdir("{}")
         if content == content_new:
             print('No changes found...')
         else:
-            save = input('Notebook changed, do you want to save changes '
-                         'in the original location? (injected parameters '
-                         'and debugging cells will be removed before '
-                         'saving). Enter "no" to skip saving changes, '
-                         'anything else will be interpreted as "yes": ')
-
             # save changes
-            if save != 'no':
+            if _save():
                 nb = nbformat.reads(content_new,
                                     as_version=nbformat.NO_CONVERT)
 
@@ -242,6 +218,39 @@ chdir("{}")
         # remove tmp file
         Path(tmp).unlink()
 
+    def debug(self, kind='ipdb'):
+        """
+        Opens the notebook (with injected parameters) in debug mode in a
+        temporary location
+        """
+        opts = {'ipdb', 'pdb'}
+
+        if kind not in opts:
+            raise ValueError('kind must be one of {}'.format(opts))
+
+        nb = _read_rendered_notebook(self.source.rendered_nb_str)
+        _, tmp_path = tempfile.mkstemp(suffix='.ipynb')
+        code = jupytext.writes(nb, version=nbformat.NO_CONVERT, fmt='py')
+        Path(tmp_path).write_text(code)
+
+        if kind == 'ipdb':
+            from IPython.terminal.debugger import TerminalPdb, Pdb
+            code = compile(source=code, filename=tmp_path, mode='exec')
+
+            try:
+                # this seems to only work in a Terminal
+                debugger = TerminalPdb()
+            except AttributeError:
+                # this works in a Jupyter notebook
+                debugger = Pdb()
+        elif kind == 'pdb':
+            debugger = pdb
+
+        try:
+            debugger.run(code)
+        finally:
+            Path(tmp_path).unlink()
+
     def run(self):
         if isinstance(self.product, MetaProduct):
             path_to_out = Path(str(self.product[self.nb_product_key]))
@@ -253,9 +262,13 @@ chdir("{}")
         # we will change the extension after the notebook runs successfully
         path_to_out_nb = path_to_out.with_suffix('.ipynb')
 
+        _, tmp = tempfile.mkstemp('.ipynb')
+        tmp = Path(tmp)
+        tmp.write_text(self.source.rendered_nb_str)
+
         try:
             # no need to pass parameters, they are already there
-            pm.execute_notebook(self.source.loc_rendered, str(path_to_out_nb),
+            pm.execute_notebook(str(tmp), str(path_to_out_nb),
                                 **self.papermill_params)
         except Exception as e:
             raise TaskBuildError('An error ocurred when calling'
@@ -263,9 +276,52 @@ chdir("{}")
                                  ' executed notebook with traceback '
                                  'available at {}'
                                  .format(str(path_to_out_nb))) from e
+        else:
+            tmp.unlink()
 
         # if output format other than ipynb, convert using nbconvert
         # and overwrite
         if self._exporter is not None:
             path_to_out_nb.rename(path_to_out)
-            _from_ipynb(path_to_out, self._exporter)
+            _from_ipynb(path_to_out, self._exporter,
+                        self.nbconvert_export_kwargs)
+
+
+def _read_rendered_notebook(nb_str):
+    """
+    Read rendered notebook and inject cell with debugging settings
+    """
+    # add debug cells
+    nb = nbformat.reads(nb_str, as_version=nbformat.NO_CONVERT)
+    nbformat_v = nbformat.versions[nb.nbformat]
+
+    source = """
+# Debugging settings (this cell will be removed before saving)
+# change the current working directory to the one when .debug() happen
+# to make relative paths work
+from os import chdir
+chdir("{}")
+""".format(Path('.').resolve())
+
+    cell = nbformat_v.new_code_cell(source,
+                                    metadata={'tags':
+                                              ['debugging-settings']})
+    nb.cells.insert(0, cell)
+
+    return nb
+
+
+def _open_jupyter_notebook(path):
+    try:
+        subprocess.call(['jupyter', 'notebook', path])
+    except KeyboardInterrupt:
+        print('Jupyter notebook server closed...')
+
+
+def _save():
+    res = input('Notebook changed, do you want to save changes '
+                'in the original location? (injected parameters '
+                'and debugging cells will be removed before '
+                'saving). Enter "no" to skip saving changes, '
+                'anything else will be interpreted as "yes": ')
+    return res != 'no'
